@@ -337,6 +337,10 @@ enum myhtml_status {
     MyHTML_STATUS_THREAD_ERROR_QUEUE_MALLOC            = 60,
     MyHTML_STATUS_THREAD_ERROR_QUEUE_NODES_MALLOC      = 61,
     MyHTML_STATUS_THREAD_ERROR_QUEUE_NODE_MALLOC       = 62,
+    MyHTML_STATUS_THREAD_ERROR_MUTEX_MALLOC            = 70,
+    MyHTML_STATUS_THREAD_ERROR_MUTEX_INIT              = 71,
+    MyHTML_STATUS_THREAD_ERROR_MUTEX_LOCK              = 72,
+    MyHTML_STATUS_THREAD_ERROR_MUTEX_UNLOCK            = 73,
     MyHTML_STATUS_RULES_ERROR_MEMORY_ALLOCATION        = 100,
     MyHTML_STATUS_PERF_ERROR_COMPILED_WITHOUT_PERF     = 200,
     MyHTML_STATUS_PERF_ERROR_FIND_CPU_CLOCK            = 201,
